#ifdef __GNUG__
#pragma implementation
#endif

#include <locale.h>
#include <stdlib.h>

#ifdef MAIN
#include <iostream>
#endif

#include "externals.h"
#include "table.h"

#include <fstream>

#ifndef NULL
#define NULL 0
#endif

// LIMITATION: the independent variable is assumed to be strictly
// increasing This is presently not checked, and if not true,
// erroneous results are silently produced.

table::table() {
  dimension = 0;
  x_data = NULL;
  y_data = NULL;
}

void
table::init(const unsigned int n, const double data[][2]) {
  dimension = n;
  //delete [] x_data;
  //delete [] y_data;
  x_data = new double[n];
  y_data = new double[n];
  for (unsigned int i = 0; i < n; i++) {
    x_data[i] = data[i][0];
    y_data[i] = data[i][1];
  }
}

void
gobble(std::ifstream *f, char separator)
{
  if (! (f->eof())) {
    char c;
    double x;
    if (separator == '\t' || separator == ' ')
      *f >> x;
    else
      do 
	*f >> c;
      while (c != separator && ! (f->eof()));
  }
}

void
eat_comment(std::ifstream *f)
{
  while (f->peek() == '#') {
    char s[10000];
    f->getline(s, 9999);
  } 
}

// Load table from csv file (first argument)
// The independent variable is found in the indep_index'th column,
// and the dependent variable is found in the dep_index'th column.
// (First column is called number 1, not 0).
// At most maxsize lines are read.
bool
table::csv_init(const char *filename,
		const unsigned int indep_index,
		const unsigned int dep_index,
		const unsigned int maxsize,
		const char separator /*= ';'*/)
{
  setlocale(LC_NUMERIC, getenv("LC_NUMERIC"));
  int no_points_read = maxsize;
  double (*array)[2];
  array = new double[maxsize][2];
  std::ifstream *f = new std::ifstream(filename);
  if (! *f) {
    gnans_error(non_fatal, "Could not open file `%s'", filename);
    return false;
  }
  
  // This case requires some more programming, too lazy now
  if (indep_index >= dep_index) {
    gnans_error(non_fatal,
		"Don't know how to handle case with independent >= dependent");
    return false;
  }

  for (unsigned int i = 0; i < maxsize && ! (f->eof()); i++) {
    double t, x;
    char c;

    eat_comment(f);
    for (unsigned int j = 0; j < indep_index - 1; j++)
      gobble(f, separator);
    *f >> t;
    if (separator != '\t' && separator != ' ')
      *f >> c;
    for (unsigned int j = indep_index; j < dep_index - 1; j++)
      gobble(f, separator);
    *f >> x;
    if (f->peek() == separator) {
      char s[1000];
      f->getline(s, 999);
    }
    if (f->eof()) {
      no_points_read = i;
      //fprintf(stdout, "-->%d\n", no_points_read);
    } else {
      //fprintf(stdout, "%g\t%g\n", t, x);
      array[i][0] = t;
      array[i][1] = x;
    }
  }
  init(no_points_read, array);
  delete f;
  delete [] array;
  return true;
}

// Load table from csv file as above, but take table index (starting
// with 0) as independent variable.
bool
table::csv_init(const char *filename,
		const unsigned int dep_index,
		const unsigned int maxsize,
		const char separator /*= ';'*/)
{
  setlocale(LC_NUMERIC, getenv("LC_NUMERIC"));
  int no_points_read = maxsize;
  double (*array)[2];
  array = new double[maxsize][2];
  std::ifstream *f = new std::ifstream(filename);
  if (! *f) {
    gnans_error(non_fatal, "Could not open file `%s'", filename);
    return false;
  }
  
  for (unsigned int i = 0; i < maxsize && ! (f->eof()); i++) {
    double x;
    char c;

    //fprintf(stdout, "$ %d ", i);
    //fprintf(stdout, f->eof() ? "EOF" : "Not EOF");
    eat_comment(f); 
    //fprintf(stdout, f->eof() ? "EOF" : "Not EOF");
    //fprintf(stdout, "C\n");
    for (unsigned int j = 0; j < dep_index - 1; j++)
      gobble(f, separator);
    //fprintf(stdout, "g\n");
    //fprintf(stdout, f->eof() ? "EOF" : "Not EOF");
    *f >> x;
    if (f->peek() == separator) {
      char s[1000];
      f->getline(s, 999);
    }
    if (f->eof()) {
      no_points_read = i;
      //fprintf(stdout, "-->%d\n", no_points_read);
    } else {
      //fprintf(stdout, "%g\n", x);
      array[i][0] = double(i);
      array[i][1] = x;
    }
  }
  init(no_points_read, array);
  delete f;
  delete [] array;
  return true;
}

double
table::value(double x) const {
  if (dimension == 0) {
    gnans_error(non_fatal, "Attempt to read from non-initialized table");
    running_integration = false;
    return 0;
  }
  if (x <= x_data[0])
    return y_data[0];
  if (x > x_data[dimension-1])
    return y_data[dimension - 1];
#ifdef STUPID
  for (unsigned int i = 0; i < dimension - 1; i++)
    if (x < x_data[i+1])
      return(y_data[i] + (y_data[i+1] - y_data[i])/(x_data[i+1] - x_data[i]) *
	     (x - x_data[i]));
  return y_data[dimension - 1];
#else
  int beg = 0, end = dimension-1;
  while (end - beg > 1) {
    int mid = (end+beg)/2;
    if (x < x_data[mid])
      end = mid;
    else
      beg = mid;
  }
  return(y_data[beg] + (y_data[end] - y_data[beg])/(x_data[end] - x_data[beg])
	 * (x - x_data[beg]));
#endif
}

double
table::slope(double x) const {
  if (dimension == 0) {
    gnans_error(non_fatal, "Attempt to read from non-initialized table");
    running_integration = false;
    return 0;
  }
  if (x <= x_data[0])
    return 0;
  if (x > x_data[dimension-1])
    return 0;
#ifdef STUPID
  for (unsigned int i = 0; i < dimension - 1; i++)
    if (x < x_data[i+1])
      return(y_data[i] + (y_data[i+1] - y_data[i])/(x_data[i+1] - x_data[i]) *
	     (x - x_data[i]));
  return y_data[dimension - 1];
#else
  int beg = 0, end = dimension-1;
  while (end - beg > 1) {
    int mid = (end+beg)/2;
    if (x < x_data[mid])
      end = mid;
    else
      beg = mid;
  }
  return (y_data[end] - y_data[beg])/(x_data[end] - x_data[beg]);
#endif
}

#ifdef MAIN

// Produces: 0 1.1 1.18 1.2
int main(int argc, char *argv[]) {
  const double junk[][2] = {{0,0}, {1,1}, {2,1.2}};
  table tab(3, junk);

  cout << tab.value(-1) << " " << tab.value(1.5) << " " << tab.value(1.9)
       << " " << tab.value(2.1) << "\n";
}

void
gnans_error(severity s, char const *fmt,...)
{
}
#endif
