/*
 Copyright  1992 Bengt Mrtensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: Euler-Maru.cc,v 1.3 1994/08/19 18:21:46 bengt Exp $

#ifdef __GNUG__
#pragma implementation
#endif

#include "Euler-Maru.h"

const char *
EulerMaruyamaIntegrator::name() const
{
  return "Euler-Maruyama";
}

const char *
EulerMaruyamaIntegrator::documentation() const
{
  return
    "Stochastic version of the explicit Euler method with fixed step-size.";
}

// This is the integator.
inline void
EulerMaruyamaIntegrator::euler_maruyama(fp f, unsigned /*number_of_states*/,
					unsigned /*number_of_wieners*/,
					double *tp, double stepsize,
					Vect &y)
{
  f(tp, y, dy);
  
  // Build a table of the increments of the wieners
  build_wiener_table(wiener_increments);

  noise = 0;
  for (unsigned j = 0; j < current_system.number_of_wieners; j++) {
    // noise += current_system.wiener_coeffs[j] * wiener_increments[j];
    current_system.wiener_coeffs[j] *= wiener_increments[j];
    noise += current_system.wiener_coeffs[j];
  }

  // y += dy * stepsize + noise*sqrt_step();
  dy *= stepsize;
  noise *= sqrt_step();
  dy += noise;
  y += dy;
  *tp += double(timestep);	// gcc bug?
}

inline bool
EulerMaruyamaIntegrator::call_integrator()
{
  euler_maruyama(current_system.f_ptr, current_system.number_of_states,
		 current_system.number_of_wieners, systemtime, timestep,
		 current_system.state_vector);
  return true;
}

bool
EulerMaruyamaIntegrator::initialize_integrator()
{
  // Assigning initial values to some internal variables
  setup_times();
    
  // Algorithm specific checks
  if (continuous_time_barf() || barf_if_step_zero() || barf_if_backwards())
    return false;

  // Algorithm independent initializations
  initialize_integration();

  // Reserve space for arrays etc.
  noise.resize(current_system.number_of_states);
  dy.resize(current_system.number_of_states);

  return true;
}
