/*
 Copyright (C) 1992 Bengt Martensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: Euler.cc,v 1.4 1994/08/19 18:21:46 bengt Exp $

#ifdef __GNUG__
#pragma implementation
#endif

#include "externals.h"
#include "Euler.h"

// This is to serve as a template for integration routine drivers

//  The function takes a boolean argument, indicating whether initialization
//  should be done or not. This argument should be true on the first call,
//  false on subsequent. Scratch variables must be static if they are to
//  retain their values between subsequent calls.
// 
//  Returns true if the integration is completely finished, false otherwise.

// This is the integrator. In this case, it is so trivial that it
// is included here, inlined.


const char *
EulerIntegrator::name() const
{
  return "Euler";
}

const char *
EulerIntegrator::documentation() const
{
  return "Explicit Euler method with fixed step-size.";
}

// It would be more natural to have the second argument double&, but for
// some reason gcc 2.7.0 breaks at that (2.6.3 manages it).

inline void
EulerIntegrator::euler(fp f, double *tp, Vect &y)
{
  f(tp, y, dy);
  dy *= signed_step;
  y += dy;
  *tp += signed_step;
}

inline bool
EulerIntegrator::call_integrator()
{
  euler(current_system.f_ptr, systemtime, current_system.state_vector);
  return true;
}

bool
EulerIntegrator::initialize_integrator()
{
  // Assigning initial values to some internal variables
  setup_times();
    
  // Algorithm specific checks
  if (continuous_time_barf() || barf_if_step_zero() || barf_on_wieners())
    return false;
    
  // Algorithm independent initializations
  initialize_integration();

  // Allocate space for arrays etc.
  dy.resize(current_system.number_of_states);
  
  return true;
}
