/* -*- c++ -*-
Copyright  1992, 1993 Bengt Mrtensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: Integrator.cc,v 1.4 1994/01/30 20:52:10 bengt Exp $

#ifdef __GNUG__
#pragma implementation
#endif

#include "externals.h"
#include "utillib.h"
#include "Integrator.h"

#ifndef HUGE
#define HUGE 3e38
#endif

// Rationale: I really don't want the plots to overshoot stoptime, since
// autoscaling plot programs produce so ugly plots if the trajectories
// end at t = 100.0001...
// I prefer the run to end at 99.99 than at 100.0000001
bool				// Inlined?
Integrator::more_to_do_nonzero_step() const
{
  return
    (signed_step > 0
     ? systemtime <= stoptime - signed_step
     : systemtime >= stoptime - signed_step)
    && running_integration
    && ! has_been_alarmed;
}

// Assumes nonzero stepsize, override in ContinuousIntegrator
bool
Integrator::finished() const
{
  return ! (starttime < stoptime ?
	    systemtime <= stoptime - signed_step
	    : systemtime >= stoptime - signed_step)
    || ! running_integration;
}

// Setting up initial time etc -- Note: NOT system_time (to make continuation
// possible)
void 
Integrator::setup_times()
{
  if (store_first_value)
    last_t_stored = starttime < stoptime ? -HUGE : HUGE;
  else
    last_t_stored = starttime;
  signed_step = starttime < stoptime ? double(timestep) : -timestep;
}

// This function stores values if last store was old enough.
void 
Integrator::store_if_necessary()
{
  if (abs(systemtime - last_t_stored) >= storeincrement) {
    store_stuff();
    last_t_stored = systemtime;
  }
}

void				// Override in discrete and riccati
Integrator::store_stuff()
{
  // Since this may be the first point after an alarm, check that we are
  // not saving the same point twice.
  // If floating_point_error_occured, don't store anything
  if (last_t_stored != systemtime && ! floating_point_error_occured) {
    last_t_stored = systemtime;
    (*current_system.f_ptr)(systemtime, current_system.state_vector,
			    junk_vector);
    (*current_system.f_store_ptr)();
    if (sample_no >= trajectories_current_size)
      resize_trajectories();

    for (register unsigned int i = 0; i < number_of_stores; i++)
      stores[i].trajectory[sample_no] = *stores[i].ptr;
    sample_no++;
  }
}

