/*
 Copyright (C) 1992 Bengt Martensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: Rkfix.cc,v 1.4 1994/08/19 18:21:46 bengt Exp $

#ifdef __GNUG__
#pragma implementation
#endif

#include "externals.h"
#include "Rkfix.h"

const char *
RkfixIntegrator::name() const
{
  return "Rkfix";
}

const char *
RkfixIntegrator::documentation() const
{
  return
    "Dumb fourth order explicit Runge-Kutta with fixed step-size.";
}

// This is the integrator.
// I try to write code that do not generate intermediate temporaties.
inline void
RkfixIntegrator::rkfix(fp f, double *t, Vect &y)
{
  f(t, y, k1);			// k1 := f(t,y)
  
  *t += step_2;

  tmp = k1;
  tmp *= step_2;
  tmp += y;
  f(t, tmp, k2);		// k2 := f(t+delta/2, y+delta*k1/2)

  tmp = k2;
  tmp *= step_2;
  tmp += y;
  f(t, tmp, k3);		// k3 := f(t+delta/2, y+delta*k2/2)

  *t += step_2;
  tmp = k3;
  tmp *= signed_step;
  tmp += y;
  f(t, tmp, k4);		// k4 := f(t+delta, y+delta*k3)

  tmp = k2;
  tmp += k3;
  tmp *=2;
  tmp += k1;
  tmp += k4;
  tmp *= step_6;

  y += tmp;			// y := y + delta/6 * (k1 + 2*k2 + 2*k3 + k4)

}

inline bool
RkfixIntegrator::call_integrator()
{
  rkfix(current_system.f_ptr, systemtime, current_system.state_vector);
  return true;
}

bool
RkfixIntegrator::initialize_integrator()
{
  // Assigning initial values to some internal variables
  setup_times();
    
  // Algorithm specific checks
  if (continuous_time_barf() || barf_if_step_zero() || barf_on_wieners())
    return false;
    
  // Algorithm independent initializations
  initialize_integration();

  // Allocate space for arrays etc.
  dy.resize(current_system.number_of_states);
  k1.resize(current_system.number_of_states);
  k2.resize(current_system.number_of_states);
  k3.resize(current_system.number_of_states);
  k4.resize(current_system.number_of_states);
  tmp.resize(current_system.number_of_states);
  
  step_2 = signed_step/2;
  step_6 = signed_step/6;

  return true;
}
