/* -*- c++ -*-
Copyright  1992, 1993 Bengt Mrtensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: SIntegrator.cc,v 1.4 1994/08/19 18:21:46 bengt Exp $

#ifdef __GNUG__
#pragma implementation
#endif

#include "SIntegrator.h"

void
StochasticIntegrator::initialize_integration()
{
  sqrt_step_internal = sqrt(timestep);
  overhead = wienertick == 0.0 ? 0 : (int) (timestep/wienertick + 0.5);
  wiener_increments.resize(current_system.number_of_wieners);
  junk_vector.resize(current_system.number_of_states);
  (*current_system.f_ptr)(systemtime, (double *) current_system.state_vector,
			  junk_vector);
  (*current_system.f_store_ptr)();
  (*current_system.f_accepted_ptr)();
}

bool
StochasticIntegrator::barf_if_backwards() const
{
  if (starttime >= stoptime) {
    gnans_error(non_fatal,
		"Integration of stochastic differential equations "
		"backwards in time is not implemented.");
    return true;
  }
  return false;
}

bool
StochasticIntegrator::scalar_noise_barf() const
{
  if (current_system.number_of_wieners != 1) {
    gnans_error(non_fatal,
		"Sorry, the selected algorithm can only handle stochastic\n"
		"differential equations with exactly ONE noise source.");
    return true;
  }
  return false;
}

bool
StochasticIntegrator::zero_wienertick_barf() const
{
  if (wienertick <= 0.0) {
    gnans_error(non_fatal,
		"The selected algorithm requires wienertick to be a positive (``small'') number");
    return true;
  }
  return false;
}
