/* -*- c++ -*-
 Copyright  1993 Bengt Mrtensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: SIntegrator.h,v 1.4 1994/08/24 09:58:06 bengt Exp $

#ifndef _SIntegrator_h
#define _SIntegrator_h

#ifdef __GNUG__
#pragma interface
#endif

#include "CIntegrator.h"
#include "externals.h"
#include "random.h"

class StochasticIntegrator : public ContinuousIntegrator {
 public:
  bool stochastic() const    { return true; }
 protected:
  inline void build_wiener_table(Vect &table);
  virtual void initialize_integration();
  Vect wiener_increments;
  double sqrt_step() const	{ return sqrt_step_internal; }
  bool barf_if_backwards() const;
  bool scalar_noise_barf() const;
  bool zero_wienertick_barf() const;
  unsigned int overhead;
 private:
  inline void build_wiener_table(double *table);
  double sqrt_step_internal;
};

inline void
StochasticIntegrator::build_wiener_table(double *table)
{
  for (unsigned int j = 0; j < current_system.number_of_wieners; j++)
    table[j] =
      current_system.wieners[j]->rng_ptr->normal(overhead);
}

inline void
StochasticIntegrator::build_wiener_table(Vect &table)
{
  build_wiener_table((double *) table);
}

#endif
