/*
 Copyright  1992, 1993 Bengt Mrtensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// Complete Reimplementation for Gnans
// (How come that the original code (to me) looks like an entry in
// the obfuscated C contest?)

// $Id: xerrwv.cc,v 1.2 1994/01/30 20:52:10 bengt Exp $

#include "externals.h"
#include "utillib.h"
#include "CIntegrator.h"

#ifdef FORTRAN_UNDERSCORES
#define xerrwv xerrwv_
#endif

#ifdef FORTRAN_UPPERCASE
#define xerrwv XERRWV
#endif

extern "C" void
xerrwv(char *msg,		// the message 
       long int *nmes,		// the length of msg (number of characters) 
       long int *nerr,		// the error number (not used) 
       long int *level,		// the error level..
				// 0 or 1 means recoverable
				// (control returns to caller)
				// 2 means fatal (run is aborted)
       long int *ni,		// number of integers (0, 1, or 2) to be
				// printed with message 
       long int *i1,		// integer to be printed, depending on ni 
       long int *i2,		// integer to be printed, depending on ni 
       long int *nr,		// number of reals (0, 1, or 2) to be
				// printed with message 
       double *r1,		// real to be printed, depending on nr 
       double *r2		// real to be printed, depending on nr 
       );

//static bool initialized = false;
//static std::string themessage = "";

// This doesn't actually "print" anything. xerrwv_flush has to be called
// at end.
 
extern "C" void
xerrwv(char *msg, long int */*nmes*/, long int */*nerr*/, long int *level,
       long int *ni, long int *i1, long int *i2,
       long int *nr, double *r1, double *r2)
{
  if (!ContinuousIntegrator::xerrwv_initialized) {
    ContinuousIntegrator::xerrwv_themessage = "";
    ContinuousIntegrator::xerrwv_initialized = true;
  }
  
  ContinuousIntegrator::xerrwv_themessage += std::string(msg) + "\n";
  
  if (*ni == 1) {
#if 0
    if (themessage.find("i1") != std::string:npos)
      themessage.substr(themessage.find_first_of("i1"), 2, number_String(*i1));
    else if (themessage.find("I1") != std::string:npos)
      themessage.replace(themessage.find("I1") = number_String(*i1);
    else
#endif
      ContinuousIntegrator::xerrwv_themessage += "\nIn above message, i1 = " + number_String(*i1) + ".";
  }
  if (*ni == 2) {
#if 0
    if (themessage.find("i1") && themessage.find("i2")) {
      themessage.at("i1", -1) = number_String(*i1);
      themessage.at("i2", -1) = number_String(*i2);
    } else if (themessage.contains("I1") && themessage.contains("I2")) {
      themessage.at("I1", -1) = number_String(*i1);
      themessage.at("I2", -1) = number_String(*i2);
    } else {
#endif
      // Did not find it
      ContinuousIntegrator::xerrwv_themessage += msg;
      ContinuousIntegrator::xerrwv_themessage += "\nIn above message, i1 = " + number_String(*i1)
	+ ", i2 = " + number_String(*i2) + ".";
#if 0
      }
#endif
  }
  if (*nr == 1) {
#if 0
    if (themessage.contains("r1"))
      themessage.at("r1") = number_String(*r1);
    else if (themessage.contains("R1"))
      themessage.at("R1") = number_String(*r1);
    else
#endif
      ContinuousIntegrator::xerrwv_themessage += "\nIn above message, r1 = " + number_String(*r1) + ".";
  }
  if (*nr == 2) {
#if 0
    if (themessage.contains("r1") && themessage.contains("r2")) {
      themessage.at("r1", -1) = number_String(*r1);
      themessage.at("r2", -1) = number_String(*r2);
    } else if (themessage.contains("R1") && themessage.contains("R2")) {
      themessage.at("R1", -1) = number_String(*r1);
      themessage.at("R2", -1) = number_String(*r2);
    } else {
#endif
      // Did not find it
      ContinuousIntegrator::xerrwv_themessage += msg;
      ContinuousIntegrator::xerrwv_themessage += "\nIn above message, r1 = " + number_String(*r1)
	+ ", r2 = " + number_String(*r2);
#if 0
      }
#endif
  }

  // abort the run if level == 2. 
  if (*level == 2) {
    gnans_error(run_fatal, "Fatal error in integration routine:\n\n%s",
		ContinuousIntegrator::xerrwv_themessage.c_str());
    ContinuousIntegrator::xerrwv_themessage = "";
  }
}

