/* -*- c++ -*-
Copyright  1993 Bengt Mrtensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: Plotter.h,v 1.6 1994/11/01 12:52:38 bengt Exp bengt $

// This file is based on work by Patrick Rosenberg 

#ifdef __GNUG__
#pragma interface
#endif

#include "externals.h"

//typedef char *String;
//#include <X11/Intrinsic.h>
extern "C" {
#include <X11/At/Plotter.h>	// Athena Plotter Widget Set V6.0
}

class PlotterControl;
class Plotter;

struct PlotterIntCallback {
  unsigned int n;
  Plotter *plotter_control;
};

class Plotter {
  // Linked list of all such Plotters
  static Plotter *head_plotter_list;
  Plotter *next;

  static bool use_color;
  
  class PlotSwitch {
  public:
      Widget widget;
      bool selected;
      const char *name;
      PlotterIntCallback callback;
      PlotSwitch() { widget = NULL; selected = false; name = NULL; };
    };

  // Who created me?
  //const PlotterControl *controller;
  
  // Pixel blob
  Pixmap mark;
   
  Widget xaxis, yaxis, form, plotShell;

  // Array of plots ("curves") as widgets
  Widget *plot;

  // Switches for determining the appearance of the plot
  bool point_p;
  bool zooming_mode;
  bool grid_p;
  bool subgrid_p;

  bool one_plot;

  unsigned int independent_index;
  unsigned int *dependents;
  unsigned int no_dependents;

  // Are all x (y) values positive (making logarithmic plots possible)?
  bool x_all_positive(double min, double max);
  bool y_all_positive(double min, double max);
  bool x_all_positive();
  bool y_all_positive();

  char title[MAXSTRINGSIZE];

  double oldxmin, oldxmax, oldymin, oldymax, minlenx, minleny;

  enum LinLogType {
    LinLin = 0,
    LinLog = 1,
    LogLin = 2,
    LogLog = 3
    };
  LinLogType current_axes_type;
  PlotSwitch linlog_table[4];
  Widget plotter, llbutton, typebutton, zoom_button, gridbutt;
  Widget subgridbutt, print_button; 
  void attach_data(const Widget parent);
  void init_linlog_switch();
  void linlog_select(LinLogType n);  
  void plot_type(bool update_lables = true, bool toggle = true);
  void grid(bool update_label = true, bool tobble = true);
  void subgrid(bool update_label = true, bool toggle = true);
  void zooming(const AtRectangleCallbackData *cad);
  void zoom();
  void print_postscript();
  void set_plot_type_label();
  void set_grid_label();
  void set_subgrid_label();
  void set_zoom_label();
  void disable();
  static void disable_plotters(Plotter *p);
  void remove_me(Plotter * &);
  void insert_in_list();
  void init(const Widget parent, unsigned int independent_index,
	    unsigned int no_dependents, unsigned int dependents[]);
public:
  static void disable_plotters();
  Plotter(const Widget parent, const PlotterControl *controller);
  Plotter(const Widget parent, unsigned int independent_index,
	  unsigned int no_dependents, unsigned int dependents[]);
  Plotter(const Widget parent, const char *var_list);
  ~Plotter();

  // Make the callback wrapper friends

friend void PlotType(Widget, XtPointer, XtPointer);
friend void Zoom(Widget, XtPointer, XtPointer);
friend void Zooming(Widget w, XtPointer client_data,
		    AtRectangleCallbackData *cad);
friend void Grid(Widget, XtPointer, XtPointer);
friend void Subgrid(Widget, XtPointer, XtPointer);
friend void PrintPostscript(Widget, XtPointer, XtPointer);
friend void Linlog_select(Widget, XtPointer, XtPointer);
friend void DeletePlotter(Widget, XtPointer, XtPointer);
friend void CreatePlotter(Widget, XtPointer, XtPointer);

friend int zzparse();
};
