/* -*- c++ -*-
Copyright  1993 Bengt Mrtensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: PlotterControl.h,v 1.7 1994/11/01 12:52:38 bengt Exp bengt $

// Based on a file written by Patrick Rosenberg 

#ifdef __GNUG__
#pragma interface
#endif

#include "externals.h"

class PlotterControl;
class PlotCandidate;

// Struct for wrapping callbacks
struct PlotterControlIntCallback {
  unsigned int n;
  PlotterControl *plotter_control;
};

class PlotterControl {
  // The head of a linked list of PlotterControl's
  static PlotterControl *head_plotter_control_list;

  // A pixel blob to mark currently selected variable
  Pixmap mark;

  // Title of popup shell
  char title[MAXSTRINGSIZE];

  // Table of stored variables
  PlotCandidate *variables;

  Widget pop;			// transientShell
  Widget plot_button;
  
  void select_dependent(const unsigned int n);
  void select_independent(const unsigned int n);

  void remove_me(PlotterControl * &);

  void insert_in_list();
  static void destroy_plotter_controls(PlotterControl *p);
  static void enable_plotter_controls(PlotterControl *p, bool enable);

  PlotterControl *next;

public: 
  static void destroy_plotter_controls();
  static void enable_plotter_controls(bool enable);
  const PlotCandidate *vars() const { return variables; }
  PlotterControl(const Widget mother_widget);
  ~PlotterControl();

// Make the callback wrapper friends

friend void Select_Dependent(Widget, XtPointer, XtPointer);
friend void Select_Independent(Widget, XtPointer, XtPointer);
friend void DeletePlotterControl(Widget, XtPointer, XtPointer);
};

// A stored UserParameter, possible to plot
class PlotCandidate {
  Widget x_wdgt;
  Widget y_wdgt;

  // Am I selected as independent variable?
  bool x_sel;

  // Am I selected to be plotted as a dependent variable?
  bool y_sel;

  // My name, in the sense of Gnans
  std::string nam;
  
  PlotterControlIntCallback callback;

public:
  PlotCandidate();

  bool x_selected() const	{ return x_sel; }
  bool y_selected() const	{ return y_sel; }

  void x_select(bool new_state) { x_sel = new_state; }
  void y_select(bool new_state) { y_sel = new_state; }
  void y_select() 		{ y_sel = ! y_sel; }

  const char *name() const	{ return nam.c_str(); }
  void setname(std::string n)   { nam = n.c_str(); }

  Widget x_widget() const	{ return x_wdgt; }
  Widget y_widget() const	{ return y_wdgt; }

friend PlotterControl::PlotterControl(const Widget mother_widget);
};
