/*   -*- c++ -*-
 Copyright (C) 1992 Bengt Martensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: externals.h,v 1.27 1994/08/19 18:38:31 bengt Exp $

// This file should contain ALL externally visible variables and function,
// with the exception of some functions declared in cmdparse.h,

// Documentation is found by definition, not in this file.

#ifndef _externals_h
#define _externals_h

#ifndef IS_GNANS_SYSTEM
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define String XtString
#include <X11/Intrinsic.h>
#undef String
#endif // ! IS_GNANS_SYSTEM

#include <string>
#include "gnans.h"		// Contains constants and defines
#include "Enums.h"
#include "Switch.h"
#include "Vect.h"
#include "Store.h"
#include "StoreFormat.h"
#include "UserParameter.h"
#include "GnansSystem.h"
#include "Symrec.h"

class Integrator;
class ContinuousIntegrator;
class DiscreteIntegrator;
#ifdef RICCATI
class RiccatiIntegrator;
#endif
/***************** globals.cc *****************/
extern bool debug_1;
extern bool debug_2;
extern bool debug_3;

#ifndef IS_GNANS_SYSTEM

extern Widget toplevel;
extern int gnansdebug;
extern unsigned int store_format;

extern bool X;
extern bool append_trajectories;
extern bool store_first_value;
#endif
extern bool store_as_default;
#ifndef IS_GNANS_SYSTEM
extern bool store_parameters;
extern bool expert_mode;
extern bool do_logging;
extern bool log_script_commands;

extern Switch *switches_table;
extern Switch switches_table_standard[];
extern Switch switches_table_development[];

extern Store *stores;

extern int zzdebug;
extern int zz_flex_debug;
extern bool malloc_debug;
extern bool trace_workproc;
extern bool trace_calls;
extern bool trace_widgets;
extern bool workprocs;
extern bool trace_algorithms;

extern bool running_integration;
extern bool running_script;
extern bool abort_script;
extern bool floating_point_error_occured;

extern bool development;
extern bool has_been_alarmed;

extern bool storeset_dirty;

extern Integrator *current_algorithm;
extern ContinuousIntegrator **algorithms;
extern DiscreteIntegrator discrete_time_integrate;
#ifdef RICCATTI
extern RiccatiIntegrator riccati_integrate;
#endif

extern StoreFormat store_format_table[];
extern std::string scriptfilename;
extern std::string system_filename;
extern std::string floating_point_format;
extern std::string default_store_format;
extern std::string default_algorithm;
extern std::string default_stochastic_algorithm;
extern double default_stochastic_timestep;
extern std::string cmd_prompt;

extern FILE *zzin;
#endif				// ! IS_GNANS_SYSTEM

// The number of these is number_of_internals (= 12)
extern UserParameter systemtime;
extern UserParameter starttime;
extern UserParameter stoptime;
extern UserParameter timestep;
extern UserParameter storeincrement;
extern UserParameter relerror;
extern UserParameter abserror;
extern UserParameter wienertick;
extern UserParameter method_flag;
extern UserParameter algorithm_parameter_1;
extern UserParameter algorithm_parameter_2;
extern UserParameter algorithm_parameter_3;

extern GnansSystem current_system;
extern unsigned int number_of_stores;
extern UserParameter *internals[number_of_internals+1];

extern unsigned sample_no;
extern unsigned trajectories_current_size;

extern std::string progname;

/**************** version.cc ****************/
extern const char version_string[];
extern const char compiled_string[];
extern const char compilation_characteristics[];

/**************** misc.cc ****************/
#ifndef IS_GNANS_SYSTEM
extern void select_algorithm(Integrator *alg);
extern void select_algorithm(unsigned int index);
extern bool select_algorithm(const char *name, bool critical);
extern bool select_algorithm(const char *name);
extern bool select_format(const char *string);
extern void die(const char *message, int errorcode);
extern "C" void MAIN_();
#endif

/**************** error.cc ****************/
extern void gnans_error(severity s, const char *fmt,...);

#ifdef HELP
/**************** help.cc ****************/
extern void help();
#endif

/**************** log.cc ****************/
extern void open_log();
extern void close_log();
extern void update_log_status();
extern void log_event(const char *str);
extern void display_in_logfile();
extern void va_log_event(const char *fmt,...);

#ifndef IS_GNANS_SYSTEM
/**************** callbacks.cc ****************/
extern void Quit(Widget widget, XtPointer client_data, XtPointer call_data);
extern void Start(Widget widget, XtPointer client_data, XtPointer call_data);
extern void Continue(Widget widget, XtPointer client_data,
		     XtPointer call_data);
extern void Stop(Widget widget, XtPointer client_data, XtPointer call_data);
extern void System(Widget widget, XtPointer client_data, XtPointer call_data);
extern void Display_vars_callback(Widget widget, XtPointer client_data,
				  XtPointer call_data);
extern void States(Widget mother_widget, XtPointer client_data,
		   XtPointer  call_data);
extern void Parameters(Widget mother_widget, XtPointer client_data,
		       XtPointer call_data);
extern void Times(Widget mother_widget, XtPointer client_data,
		  XtPointer call_data);
extern void Functions(Widget mother_widget, XtPointer client_data,
		      XtPointer call_data);
extern void Seeds(Widget mother_widget, XtPointer client_data,
		  XtPointer call_data);
extern void Settings(Widget mother_widget, XtPointer client_data,
		     XtPointer call_data);
extern void Continue(Widget mother_widget, XtPointer client_data,
		     XtPointer call_data);
extern void Help(Widget mother_widget, XtPointer client_data,
		 XtPointer call_data);
extern void About(Widget mother_widget, XtPointer client_data,
		 XtPointer call_data);
extern void Script(Widget mother_widget, XtPointer client_data,
		   XtPointer call_data);
extern void Plot(Widget mother_widget, XtPointer client_data,
		 XtPointer call_data);
extern void popup_error_message(Widget mother_widget, XtPointer client_data,
				XtPointer the_error_message,
				const char */*XtPointer*/ the_title, XtGrabKind grab_type);
extern void destroy_toplevel();
extern void destroy_inquiry_popup(Widget mother_widget, XtPointer client_data,
				  XtPointer call_data);
extern void popup_position(Widget popup, Widget parent,
			   XtGrabKind grab_type, bool disable_parent);
extern void change_value_action(Widget widget, XEvent *event, XtString *params,
				Cardinal *num_params);
extern void destroy_popup_action(Widget widget, XEvent *event,
				 XtString *params, Cardinal *num_params);
extern char *make_popup_title(char *title, const char *name);
extern void zap_popups();
extern void make_equal_width(Widget w1, Widget w2);

/**************** fileselector.cc ****************/
extern void fileselector(char *title, char *name_of_label_widget, char *ending,
			 char *filename_ptr, void (*run_fcn)(char *str),
			 void (*message_fcn)(char *str),
			 bool must_exist, bool empty_ok);
extern void fileselector_open_action(Widget widget, XEvent *event,
				     XtString *params, Cardinal *num_params);

/**************** debug.cc ****************/
extern void debug_setup();
extern void update_development_status();

/**************** main.cc ****************/
extern int main(int argc, char *argv[]);
extern void destroy_toplevel();
extern void register_work_proc(bool continuation,
			       bool (*proc)(bool initialize) = NULL);
extern void disable_some_buttons();
extern void enable_buttons_idle();
extern void enable_buttons_busy();
extern void set_toplevel_title();
extern void reset_cpu();
extern void indicate_progress();
extern void enable_busy_window();
extern void disable_busy_window();

/**************** integrate.cc ****************/
extern bool integrate(Continuation continuation);
extern bool continue_integration(bool continuation);
extern void integrate_finish();
extern void setup_system_static();
extern void write_trajectories();
extern void resize_trajectories();
extern void append_trajectories_hook();
#ifdef DYNAMIC_LOADING
extern void setup_system_dynamic(std::string so_filename);
#endif

/**************** signals.cc ****************/
extern void signal_setup();
extern void set_alarm();
extern void cancel_alarm();
extern void update_workproc_status();

/**************** commands.cc ****************/
extern void do_prompt();
extern void decode_commands(const char *scriptfilename);
extern void store(const char *name, bool stored);
extern void toggle_switch(const char *name, bool value);
extern void zzerror(char *s);
extern void update_symbol_table();

/**************** save_state.cc ****************/
extern int save_state(const char *name);
extern int save_state_numbered(const char *name, int index);
extern void restore_state(const char *name, int index);

/**************** display.cc ****************/
extern void Display_vars(int code);
extern char *create_display_string(int code);
extern void emergency_dump();

/**************** yacc/lex stuff ****************/
extern int zzparse();
extern int zzlex();
extern void zzrestart(FILE *zzin);
#endif				// ! IS_GNANS_SYSTEM

/**************** f.cc ****************/
// This is linked in with the dynamic loader. Therefore, try to make
// life as simple as possible for the dynamic loader. I.e. avoid 
// C++ linkage and elaborate return types.
// In particular, we use a return type of int instead of bool.
extern "C" int gnans_init(double y[], Continuation continuation,
			  bool stochastic);
extern "C" void gnans_finish();
extern "C" int gnans_setup(double **x, double *t);
extern "C" void gnans_f(double *t, double *x, double *dx);
extern "C" void gnans_discrete_f();
extern "C" void gnans_f_accepted();
extern "C" void gnans_f_store();
extern "C" void gnans_destroy();

/**************** ../integrators/setup.cc ****************/
extern void integrators_setup();

/** Previously cmdparse.h **/
extern symrec *putsym(const char *sym_name, int sym_type, bool ro);
extern symrec *getsym(const char *sym_name);
extern void zzerror(char *s);

#endif				// ! _externals_h
