/*
 Copyright (C) 1992 Bengt Martensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

/* $Id: save_state.cc,v 1.5 1993/05/23 13:21:09 bengt Exp $ */

/*
 * This file defines the function save_state, restore_state etc, which
 * are used to save and restore the states of the rng's from the parser.
 */

#include "externals.h"
#include "random.h"

#define STR_EQUAL(s1, s2) ! strcmp((s1), (s2))

static UserParameter
*name2userpar(const char *name)
{
  for (unsigned int i = 0; i <= current_system.number_of_randoms - 1; i++)
    if (STR_EQUAL(current_system.randoms[i]->name(), name))
      return current_system.randoms[i];
  return NULL;
}


extern int
save_state(const char *name)
{     
  UserParameter *up = name2userpar(name);
  if (! up) {
    gnans_error(parse, "No random number generator named %s", name);
    return -1;
  }
  return ((rng *) up->rng_ptr)->save_state();
}

extern int
save_state_numbered(const char *name, int index)
{     
  UserParameter *up = name2userpar(name);
  if (! up) {
    gnans_error(parse, "No random number generator named %s", name);
    return MAX_SAVES;
  }
  return ((rng *) up->rng_ptr)->save_state(index);
}

extern void
restore_state(const char *name, int index)
{
  UserParameter *up = name2userpar(name);
  if (! up)
    gnans_error(parse, "No random number generator named %s", name);
  ((rng *) up->rng_ptr)->restore_state(index);
}
