/*
 Copyright (C) 1992 Bengt Martensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

/* $Id: va_funcs.cc,v 1.13 1994/08/19 18:19:37 bengt Exp $ */

/*
 * Some more utility functions. (The filename is somewhat obsolete.)
 */

#include <config.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include "translator.h"

static void die(const char *message, int errorcode);

/*
 * Generates a string from the arguments, which should be like in printf
 * Note: This is not a general purpose function for turning C into Lisp.
 * The user should delete the return argument.
 */
extern char 
*stringify(const char *format,...)
{
  va_list args;
  va_start(args, format);
  char str[BIGSTRINGSIZE];
  vsprintf(str, format, args);
  va_end(args);
  char *s = new char[strlen(str)+1];
  strcpy(s, str);
  return s;
}

const unsigned int max_errors = 100;
static void
bump_error_occured()
{
  errors_occured++;
  if (errors_occured == max_errors)
    gnans_error(fatal, "That makes %d errors, please try again.", max_errors);
}

/**************** General purpose error reporting function ****************/
extern void 
gnans_error(severity sev, const char *format, ...)
{
  char mess1[BIGSTRINGSIZE], mess2[BIGSTRINGSIZE];
  va_list args;
  va_start(args, format);
  vsprintf(mess1, format, args);

  switch (sev) {
  case parse:
    fprintf(stderr, "%s:%d:parse error\n", input_filename.c_str(),
	    lineno);
    fprintf(stderr, "Last read line: %s\n", current_line);
    bump_error_occured();
    break;
  case semantic:
    fprintf(stderr, "%s:%d:error: %s\n", input_filename.c_str(),
	    lineno, mess1);
    fprintf(stderr, "Last read token(s): %s\n", current_line);
    bump_error_occured();
    break;
 case recoverable:		// Successfull translation not possible
    if (mess1[0])
      sprintf(mess2, "%s: %s\n", input_filename.c_str(), mess1);
    else
      strcpy(mess2, "");
    fprintf(stderr, mess2);
    bump_error_occured();
    break;
  case fatal:
    if (mess1[0])
      sprintf(mess2, "%s: %s\n", input_filename.c_str(), mess1);
    else
      strcpy(mess2, "");
    die(mess2, fatal);
    break;
  case impossible:
    fprintf(stderr, "%s\n", mess1);
    abort();
    break;
  default:
    fprintf(stderr, "Internal error, dumping core\n");
    abort();
    break;
  }
}

static void 
die(const char *message, int errorcode)
{
  fclose(outputfile);
  if (! translator_debug)
    unlink(output_filename.c_str());
  if (message)
    fprintf(stderr,message);
  exit(errorcode);
}
