/*
 Copyright (C) 1992 Bengt Martensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

/* $Id: utils.cc,v 1.6 1994/08/19 18:14:03 bengt Exp $ */

/* This file defines a bunch of utility functions */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <sys/param.h>

#include <time.h>
#include <stdarg.h>
#ifdef HAVE_IEEE754_H
#include <ieee754.h>		// For NaN

#else  // ! HAVE_IEEE754_H

#if !defined(__BIG_ENDIAN) && !defined(__LITTLE_ENDIAN)
#ifdef WORDS_BIGENDIAN
#define __BIG_ENDIAN
#else
#define __LITTLE_ENDIAN
#endif
#endif

union ieee754_double
  {
    double d;
    
    /* This is the IEEE 754 double-precision format.  */
    struct
      {
#ifdef  __BIG_ENDIAN
        unsigned int negative:1;
        unsigned int exponent:11;
        /* Together these comprise the mantissa.  */
        unsigned int mantissa0:20;
        unsigned int mantissa1:32;
#endif
#ifdef  __LITTLE_ENDIAN
        /* Together these comprise the mantissa.  */
        unsigned int mantissa1:32;
        unsigned int mantissa0:20;
        unsigned int exponent:11;
        unsigned int negative:1;
#endif
      } ieee;
  };

#endif // ! HAVE_IEEE754_H

#ifndef HAVE_GETHOSTNAME
//#include <sys/systeminfo.h>
#include <sys/sysinfo.h>
#endif

#ifdef __GNUG__
#pragma implementation "utillib.h"
#endif
#include "utillib.h"

#ifndef GETHOSTNAMEDECLARED
//**//extern "C" int gethostname(char *name, int namelen);
#endif

const int MAXSTRINGSIZE = 1000;
#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 256
#endif

/****************************************************************/
/**************** General utility functions ****************/

/* Destructively modify the so_filename so that the ending is removed */
extern void 
zap_suffix(char *so_filename)
{
  if ((! strrchr(so_filename, '/')
       || strrchr(so_filename, '.') > strrchr(so_filename, '/'))
      && strrchr(so_filename, '.'))
    *strrchr(so_filename, '.') = '\0';
}

/* This function returns NaN. */
extern double
gnans_NaN()
{
  union ieee754_double u;
  u.ieee.exponent = 0x7ff;
  u.ieee.mantissa0 = 0xfffff;
  u.ieee.mantissa1 = 0xffffffff;
  u.ieee.negative = 0;
  return u.d;
}

/* Returns the date and time. */
extern char 
*getdate()
{
  int len;
  long t;
  char *p;

  t = time((long *) 0);
  p = ctime(&t);
  len = strlen(p);
  *(p + len - 1) = '\0';  /* zap the newline character */
  return p;
}

// /* This is identical to basename(1) */
// extern const char *
// basename(const char *fullname)
// {
//   const char *slash = strrchr(fullname, '/');
//   return slash ? slash + 1 : fullname;
// }

// /* This works like dirname(1) */
// extern char *
// dirname(char *dir, char *fullname)
// {
//   char *slash;
//   slash = strrchr(fullname, '/');
//   if (slash == NULL)
//     strcpy(dir, "");
//   else {
//     strcpy(dir, fullname);
//     slash = strrchr(dir, '/');
//     *slash = '\0';
//   }
//   return dir;
// }

#if 0
/*
 * Generates a String from the arguments, which should be like in printf.
 */
extern String
Stringify(const char *format,...)
{
  const unsigned int MAXSTRINGSIZE = 1000;
  va_list args;
  va_start(args, format);
  char *str = new char[MAXSTRINGSIZE];
  vsprintf(str, format, args);
  va_end(args);
  String s(str);
  delete [] str;
  return s;
}
#endif

//
extern std::string
number_String(double x)
{
  char tmp[MAXSTRINGSIZE];
  sprintf(tmp, "%g", x);
  std::string s(tmp);
  return s;
}

extern std::string
number_String(int x)
{
  char tmp[MAXSTRINGSIZE];
  sprintf(tmp, "%d", x);
  std::string s(tmp);
  return s;
}

extern std::string
number_String(long int x)
{
  return number_String(int(x));
}

extern std::string
number_String(unsigned int x)
{
  return number_String(int(x));
}

// gethostname is not declared in any includes; and there is a warning.
// The GNU coding standards recommends NOT to define it. This makes sense.
extern std::string
gethostname_as_string()
{

#ifdef HAVE_GETHOSTNAME
  char thehostname[MAXHOSTNAMELEN + 1];
  gethostname(thehostname, MAXHOSTNAMELEN);
#else
  char thehostname[MAXSTRINGSIZE];
  sysinfo(SI_HOSTNAME, thehostname, MAXSTRINGSIZE);
#endif
  std::string s(thehostname);
  return s;
}

#ifdef MAIN
int extern
main(int argc, char *argv[])
{
  printf("%g\n", gnans_NaN());
  return 0;
}
#endif
